/*
  Generates 4096x4096 points "sine wave" on Arduino Due PORTC pins.
  By MrWolf@EEVblog forum

  33 C1 D0
  34 C2 D1
  35 C3 D2
  36 C4 D3
  37 C5 D4
  38 C6 D5
  39 C7 D6
  40 C8 D7

  51 C12 D8
  50 C13 D9
  49 C14 D10
  48 C15 D11
*/

#define NOP __asm__ __volatile__ ("nop\n\t");

const int iPoints = 4096;
unsigned int aiWave[4096];

void setup()
{
  REG_PIOC_OWER = 0xFFFFFFFF;
  REG_PIOC_OER = 0xFFFFFFFF;

  float fPhaseStep = 2 * PI / iPoints;
  float fPhase = -PI;
  float fY;
  int i;
  unsigned int x, x1, x2;

  for (i = 0; i < iPoints; i++)
  {
    fY = sin(fPhase);
    fY = fY * 2047;
    x = (int) fY + 2048;
    x1 = x & 0x000000FF;
    x1 = x1 << 1;
    x2 = x & 0x00000F00;
    x2 = x2 << 4; 
    x = x1 | x2;
    if (i == 0) x = x + 0x10000; // Trigger @C16
    aiWave[i] = x;
    fPhase += fPhaseStep;
  }
}

void loop()
{
  int i;
  for (i = 0; i < iPoints; i++)
    REG_PIOC_ODSR = aiWave[i];
}
